package eu.clarin.cmdi.mdservice.model;

/**
 * The main model class, carrying users_query
 * able to parse and transform the user-string
 * and execute the query, ie issue appropriate request on the MDRepository
 * getting the url-wording right from MDRepositoryProxy
 *   
 * At the moment just responsible for querying MDRepository
 * perhaps unify approach for other proxies - subtyping this Class  
 */

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.io.IOUtils;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.z3950.zing.cql.CQLNode;
import org.z3950.zing.cql.CQLParseException;
import org.z3950.zing.cql.CQLParser;

import eu.clarin.cmdi.mdservice.internal.Admin;
import eu.clarin.cmdi.mdservice.internal.MDTransformer;

public class Diagnostic {	
	public static int SYSTEM_ERROR = 1;
	public static int UNSUPPOERTED_OPERATION = 4;
	public static int UNSUPPORTED_VERSION = 5;
	public static int UNSUPPORTED_PARAMETERVALUE = 6;
	public static int MANDATORY_NOTSUPPLIED = 7;
	public static int UNSUPPORTED_PARAMETER = 8;
	public static int QUERYSYNTAXERROR = 10;
	
	// types ?
	public static String FATAL = "fatal";
	public static String NONFATALSURROGATE = "surrogate";
	public static String NONFATALNONSURORGATE = "nonsurrogate";
	
	public static String NAMESPACE = "info:srw/diagnostic/1/";
	
	private String type;
	
	private int number;
	private String details;  
	private String message;
	
	public int getNumber(){
		return this.number;
	}
	
	public void setNumber(int number){
		this.number = number;
	}
	
	public String getType(){
		return this.type;
	}
	
	public void setType(String type){
		this.type = type;
	}
	
	public String Type(){
		String t = this.type;
		
		if (t == null){
			t = Diagnostic.diagnostictypes.get(this.number);
			if (t == null){
				t = FATAL;
			}
		}
		return t;
	}
	public Diagnostic(int number) {
		this.number = number;
		this.message = diagnosticmessage.get(number);
		//this.type = null;
	}

	public Diagnostic(int number, String details) {
		this(number,details,null);
	}

	public Diagnostic(int number, String details, String type) {
		this(number);
		this.details = details;
		this.type = type;
	}

	public Diagnostic(int number, String details, String type,String message) {
		this.number = number;
		this.details = details;
		this.type = type;
		this.message = message;
	}
	
	private final static HashMap<Integer,String> diagnosticmessage = new HashMap<Integer,String>();
	 static
	 {	diagnosticmessage.put(SYSTEM_ERROR, "General system error.");
	 	diagnosticmessage.put(UNSUPPOERTED_OPERATION, "Unsupported operation.");
	 	diagnosticmessage.put(UNSUPPORTED_VERSION, "Unsupported version.");
	 	diagnosticmessage.put(UNSUPPORTED_PARAMETERVALUE, "Unsupported parameter value.");
	 	diagnosticmessage.put(MANDATORY_NOTSUPPLIED, "Mandatory parameter not supplied.");
	 	diagnosticmessage.put(UNSUPPORTED_PARAMETER, "Unsupported Parameter.");
	 	diagnosticmessage.put(QUERYSYNTAXERROR, "Query syntax error.");

	 } 
	 public final static HashMap<Integer,String> diagnostictypes = new HashMap<Integer,String>();
	 static
	 {	diagnostictypes.put(SYSTEM_ERROR, FATAL);
	 	diagnostictypes.put(UNSUPPOERTED_OPERATION, FATAL);
	 	diagnostictypes.put(UNSUPPORTED_VERSION, FATAL);
	 	diagnostictypes.put(UNSUPPORTED_PARAMETERVALUE, FATAL);
	 	diagnostictypes.put(MANDATORY_NOTSUPPLIED, FATAL);
	 	diagnostictypes.put(UNSUPPORTED_PARAMETER, FATAL);
	 	diagnostictypes.put(QUERYSYNTAXERROR, FATAL);

	 } 
	 
	public Element buildXMLNode(Document doc){
		
		Element d = doc.createElement("diagnostic");
		Attr attr = doc.createAttribute("xmlns");
		attr.setValue("http://www.loc.gov/zing/srw/diagnostic/");
		d.setAttributeNode(attr);

		Element uri = doc.createElement("uri");
		uri.appendChild(doc.createTextNode(NAMESPACE + Integer.toString(number)));
		d.appendChild(uri);
		
		if (this.details != null){
			Element details = doc.createElement("details");
			details.appendChild(doc.createTextNode(this.details));
			d.appendChild(details);
		}
		
		if (this.message != null){
			Element message = doc.createElement("message");
			message.appendChild(doc.createTextNode(this.message));
			d.appendChild(message);
		}
		return d;
		  
	}
}
