package eu.clarin.cmdi.mdservice.proxy;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import eu.clarin.cmdi.mdservice.action.GenericAction;
import eu.clarin.cmdi.mdservice.action.RepoAction;
import eu.clarin.cmdi.mdservice.internal.Admin;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;

/**
 * Base class for implementation of proxies to target systems.
 * This is now (after rework 2011-09) decoupled from the called Action. 
 * 
 * Defines the methods used for retrieving the data 
 * and provides basic implementation where possible.
 *
 *  
 * @author vronk
 *
 */
public class BasicProxy implements ProxyInterface{

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("BasicProxy");
 
	private String proxy_key = "basic";
	
	private GenericAction source_action;
	
	public GenericAction getSourceAction() {
			return source_action;
	}
	
	@Override
	public void setSourceAction(GenericAction action) {		
		source_action = action;
		
	}
	
	/**
	 * Convenience function to access the request-parameters in the Action  
	 * @param key
	 * @return
	 */
	public String getParam(String key) {
		return getSourceAction().getParam(key);
	}
	
	public void checkParams() {
		// required Params	
	}
	
	
	/**
	 * identification of the target-proxy (type)
	 * base for finding the right base_url in the props
	 * subclass has to override with its specific proxykey
	 * @return
	 */
	public String getProxyKey() {				
			return proxy_key;		
	}

	/**
	 * Get the base_uri based on the action-key
	 */
		
	public String getBaseURI() {		
		String uri =Admin.getConfig().getProperty(getSourceAction().getActionkey() + ".uri");				
		return uri;
	}

	/**
	 * Provides the base-URL specific to given target repository or registry 
	 * @return 
	 * @throws MalformedURLException
	 */	
	public URL getBaseURL() throws MalformedURLException {		
		URL base_url = new URL(getBaseURI());		
		return base_url;
	}
	
	
	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws Exception 
	 */	
	public URL getTargetRequest() throws IOException {
		 
		URL targetURL =getBaseURL();
		
		log.debug("getTargetRequest().targetURL:" + targetURL);
	    return targetURL;
		
	}

	public InputStream getSourceStream() throws IOException, NoStylesheetException {
		return getTargetRequest().openStream();		
	}

	

}	