package eu.clarin.cmdi.mdservice.proxy;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;

/**
 
 */
public class Pz2Proxy extends RepoProxy {

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("Pz2Proxy");
	
	private String proxy_key = "pazpar2";
	
	private String command;
	private String query;
	private String sessionID;
	private String sort;
	private String block;
	
	public void setCommand(String command) {
		this.command = command;
	}
	public String getCommand() {
		return command;
	}
	public void setQuery(String query) {
		this.query = query;
	}
	public String getQuery() {
		return query;
	}
	public void setSessionID(String sessionID) {
		this.sessionID = sessionID;
	}
	public String getSessionID() {
		return sessionID;
	}

	public void setSort(String sort) {
		this.sort = sort;
	}
	public String getSort() {
		return sort;
	}
	public void setBlock(String block) {
		this.block = block;
	}
	public String getBlock() {
		return block;
	}
	
	
	public String createURLParams(){
		String params = "?";
		if (!(command == null)){
			params = params + "command=" + command;
		}
		if (!(sessionID == null)){
			params = params + "&session=" + sessionID;
		}
		if (!(query == null)){
			params = params + "&query=" + query;
		}
		/*
		  if (!(getSourceAction().getStartItem() == null)){
		 
			params = params + "&start=" + getSourceAction().getStartItem();
		}
		if (!(getSourceAction().getMaximumItems() == null)){
			params = params + "&num=" + getSourceAction().getMaximumItems();
		}
		*/
		if (!(sort == null)){
			params = params + "&sort=" + sort;
			if (!(block == null)){
				params = params + ":" + block;
			}
		}
		
		return params;
	}
	@Override
	public String getBaseURI() {		
		String uri = WorkspaceProfile.getRepositoryPath(getSourceAction().getRepository());		
		return uri;
	}
	
	@Override
	public URL getTargetRequest() throws IOException {
        URL targetURL = null;		
			
		targetURL =new URL( getBaseURL(), createURLParams() );
		log.debug("Pz2.targetURL:" + targetURL);
	    return targetURL;
	}
	
/**
 * Overrides the default implementation for the case, when the Service shall act as "collector",
 * waiting for the final result (implemented in getSourcePz2
 * The distinction is based on the command parameter: if it is given, the Class(Service) acts as a basic proxy
 * transparently passing the communication between client and target repository.  
 */
	@Override
	public InputStream getSourceStream() throws IOException, NoStylesheetException {
		if (getCommand() == null){
			try {
				return getSourcePz2();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} 
			
		return super.getSourceStream();
	}
	
/**
 * TODO: move to Utils
 * @param is
 * @return
 */
	public Document getDocument(InputStream is){
		Document doc = null;
		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
     	DocumentBuilder docBuilder;
		try {
			docBuilder = docFactory.newDocumentBuilder();
			doc = docBuilder.parse(is);
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return doc;
	}
	
/**
 * TODO: move to Utils
 * @param is
 * @param expression
 * @return
 */
	public String getDocumentData(InputStream is, String expression){
		String sessionid = "";
	
		XPathFactory factory = XPathFactory.newInstance(); 
	    XPath xpath = factory.newXPath(); 
	    XPathExpression expr;
		try {
			expr = xpath.compile(expression);
			sessionid = (String) expr.evaluate(getDocument(is), XPathConstants.STRING);
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		        
		return sessionid;
		
	}

/**
 * This function wraps the session-based communication with a Pazpar2-target, waiting until all the responses were returned (or timeout occured)
 * returning the full result at the end.   
 * @return returns the collected final result as a stream
 * @throws Exception
 */
	public InputStream getSourcePz2() throws Exception {
		
		InputStream is = null;
		// TODO param settings
		//init
		setCommand("init");
		String sessionID = getDocumentData(this.getSourceStream(), "//init/session");
		// get result
		setCommand("search");
		setSessionID(sessionID);
		//setQuery(this.getSquery()); //matej 20110903
		setQuery(this.getQuery());
		String ok = getDocumentData(this.getSourceStream(),"//search/status");
		
		String activeclients = "1";
		setCommand("show");
		setSort("relevance");
		setBlock("1");
		//TODO timeout  settings
		long startMillis = System.currentTimeMillis();
		long timeout = 0;
		if (WorkspaceProfile.getOption("pazpartimeout") == null){
			timeout = 1000;
		} else {
			timeout = Integer.parseInt(WorkspaceProfile.getOption("pazpartimeout"));
		}
		while (Integer.parseInt(activeclients) > 0){
			//activeclients = getDocumentData(this.getSourceStream(), "//show/activeclients");
			is = this.getSourceStream();
			activeclients = getDocumentData(is, "//show/activeclients");
			Thread.sleep(1000); 
			log.debug("ActiveClients: " + activeclients);
			if ((System.currentTimeMillis() - startMillis)/1000 > timeout) {
				break;
			}
		}
		is = this.getSourceStream();
		return is;
	}
	
}